# Generated by Django 4.2.6 on 2023-10-10 14:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0003_user_last_message_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='want_action',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
    ]
