# Generated by Django 4.2.6 on 2023-10-09 18:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message_id', models.BigIntegerField()),
                ('update_id', models.BigIntegerField()),
                ('from_id', models.BigIntegerField()),
                ('text', models.TextField(blank=True, null=True)),
                ('data', models.TextField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('t_id', models.CharField(default='', max_length=100, verbose_name='Telegram ID')),
                ('username', models.CharField(default='', max_length=200, verbose_name='Username')),
                ('name', models.CharField(default='', max_length=200, verbose_name='Имя')),
            ],
        ),
    ]
