# Generated by Django 4.2.6 on 2023-10-10 10:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0002_message_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='last_message_id',
            field=models.BigIntegerField(blank=True, null=True, verbose_name='Номер последнего сообщения'),
        ),
    ]
